;adda-eeprom-messwertspeicher.asm ****************************************************************************
;			10 Analogwerte pro Sekunde speichern im int. EEPROM, anschlieend wieder ausgeben 
;			Aufnahmezeit ca 25 Sekunden, da 256 Werte gespeichert werden
;			Analogwert: Poti 0 bis 5V an Ain0
;			Zum Einlesen neuer Werte Taster P3.2 und RESET gleichzeitig drcken, zuerst Reset loslassen
;			Digitalwert-Ausgabe: LEDs P2
;*** Achtung: Nicht zu oft neue Werte einlesen, da Anzahl Schreibzyklen auf das EEPROM begrenzt (Lebensdauer)
;*************************************************************************************************************
		code at 0			;Programmanfangsadresse
		include REG_8252.pdf	;Registeradressen AT89S8252
		
		TasterEin	bit	p3.2	;Taster mu bei Reset gedrckt sein um neue Werte einzulesen
		
;********** Hilfsprogramme in ADDAhilf.a51 *******************************************************************
		extern code Ain0,Aout	;Datei addahilf im Ordner hilf_asm zum Projekt hinzufgen !

;********** Register WMCON zur Steuerung des internen EEPROMS ************************************************			
;		WMCON				;Bit4:	1 int EEPROM zum Schreiben freigeben
						;Bit3:	1 int EEPROM mit movx ansprechen
						;Bit1:	ist 0 wenn int.EEPROM-Schreibzyklus noch nicht beendet

;********** Programm *****************************************************************************************
		orl	WMCON,#00001000b	;int EEPROM durch movx ansprechen
		jnb	TasterEin,ausDA	;Sprung wenn Taster nicht gedrckt zur Ausgabe der gespeicherten Werte
		
	; Einlesen ---------------------------------------------------------------------------------------------
		orl	WMCON,#00010000b	;int EEPROM zum Schreiben freigeben	
		mov	dptr,#0		;Anfangsadresse EEPROM
loopAD:	lcall	Ain0			;Wert vom Analogeingang 0 holen
		movx	@dptr,a		;im EEPROM ablegen
		mov	p2,a			;und an LEDs an P2 ausgeben
		
waitEEPROM:	mov	a,WMCON		;Zustand Schreibzyklus holen
		anl	a,#00000010b	;Bit1 maskieren (s.o.)
		jz	waitEEPROM		;Sprung wenn int. EEPROM-Schreibzyklus noch nicht beendet
	
		inc	dptr			;nchste Speicherstelle
		mov	a,dpl
		jz	ausDA			;Sprung zur Ausgabe wenn schon 256 Speicherstelle
		lcall	zeit			;sonst Zeitverzgerung
		sjmp	loopAD		;nchsten Wert holen und speichern
		
	; Ausgeben ---------------------------------------------------------------------------------------------
ausDA:	mov	dptr,#0		;Anfangsadresse EEPROM
loopDA:	movx	a,@dptr		;Wert holen
		lcall	Aout			;und ausgeben an Analogausgang
		mov	p2,a			;und LEDs
		lcall	zeit			;Zeitverzgerung
		inc	dptr			;nchste Speicherstelle
		mov	a,dpl
		jz	ausDA			;von vorne alle 256 Werte ausgeben
		sjmp	loopDA		;einer der 256 Werte
		
	; Zeitverzgerung --------------------------------------------------------------------------------------
		mov	r0,#250
		mov	r1,#200
zeit:		djnz	r0,zeit		;Zeitverzgerung ca. 250 x 200 x 2s = 100ms
		djnz	r1,zeit
		ret
end

